#!/usr/bin/perl

################################################################
# The following two functions are part of resolv.conf watcher
# Due to DHCP, resolv.conf sometimes gets updated with new entries 
# of DNS servers, and in such event, the watcher restarts CoDNS.
#
# Warning: This script has to reside in the CoDNS directory
################################################################

sub LinuxWatcher
{
    # watches the change in resolv.conf and restarts CoDNS
    # this includes Linux-like systems
    my $last_mod_time = 0;
    my $updated = 0;
    my $resolvfile = "/etc/resolv.conf";

    if (-e $resolvfile) {
	$last_mod_time = (stat $resolvfile)[9];
	sleep(1);
    }
    for (;;) {
	if (-e $resolvfile) {
	    my $cur_last_mod_time = (stat $resolvfile)[9];

	    if( $last_mod_time != $cur_last_mod_time) {
		# check resolv.conf has the entry, "127.0.0.1"
		# if not, restart CoDNS
		my ($good, $first) = (0, 1) ;

		open THIS_FILE, $resolvfile;
		while(<THIS_FILE>) {
		    if (/nameserver\s+127.0.0.1/) {
			$good = 1;
		    } elsif (/nameserver/) {
			last;
		    }
		}
		close THIS_FILE;
		
		if ($good == 0) {
		    system("./codns_start -r");
		}
		$last_mod_time = $cur_last_mod_time;
	    }
	    $updated = 1;
	}
	
	if ($updated) {
	    sleep(60);
	} else {
	    sleep(1);
	}
    }
}
################################################################

sub BSDWatcher
{
    my $last_mod_time = 0;
    my $updated = 0;
    my $resolvfile = "/var/run/resolv.conf";

    if (-e $resolvfile) {
	$last_mod_time = (stat $resolvfile)[9];
	sleep(1);
    }
    for (;;) {
	if (-e $resolvfile) {
	    my $cur_last_mod_time = (stat $resolvfile)[9];

	    if( $last_mod_time != $cur_last_mod_time) {
		# something has been changed in "resolv.conf"
		system("./codns_start -r");
		$last_mod_time = (stat $resolvfile)[9];
	    }
	    $updated = 1;
	}
	
	if ($updated) {
	    sleep(60);
	} else {
	    sleep(1);
	}
    }
}

################################################################
# CoDNS Log compressor
################################################################

# The script starts here

# spawn resolv.conf watcher
if (fork() == 0) {
    my $uname;
    
    chomp($uname = `uname`);
    if ($uname eq "Darwin" || ($uname =~ /"BSD"/)) {
	BSDWatcher();
    } else {
	LinuxWatcher();
    }
    exit(0);
}

################################################################

my $dir;
my $next_log;
my $date;

chomp($dir = `pwd`);
print "current directory=$dir\n";

$dir = "$dir/log" if (-d "$dir/log");
for (;;) {
    $date = `date -u +%Y%m%d`;
    chomp($date);

    open H_PIPE, "ls $dir | grep . |";
    while (<H_PIPE>) {
        chomp; # remove '\n'
        if (/(dbgcodns)\.(........)_(...)$/) {
            `bzip2 -f $dir/$_`;
        }
    }
    close H_PIPE;

    # wait 5 minutes
    sleep(5*60);
}
