#include "codnslib.h"
#include "codns_int.h"
#include "codnshbt.h"

#define BITMASK 0xFFFFFFF
#define _rotl(Val, Bits) \
       ((((Val)<<(Bits)) | (((Val) & BITMASK)>>(32 - (Bits)))) & BITMASK)

/* Common Library */
/*-----------------------------------------------------------------*/
double
DecayAvg(double oldAvg, double sample, int *count, int countMax)
{
  double newAvg;
  int newCount = *count + 1;

  newAvg = (*count * oldAvg + sample) / newCount;
  if (newCount >= countMax)
    newCount = countMax-1;
  *count = newCount;
  return(newAvg);
}
/*-----------------------------------------------------------------*/
struct timeval
AddTimes(struct timeval a, struct timeval b)
{
  a.tv_sec += b.tv_sec;
  a.tv_usec += b.tv_usec;
  if (a.tv_usec > 1e6) {
    a.tv_usec -= 1e6;
    a.tv_sec++;
  }
  return a;
}
/*-----------------------------------------------------------------*/
struct timeval
DiffTimes(struct timeval a, struct timeval b)
{
  a.tv_sec -= b.tv_sec;
  a.tv_usec -= b.tv_usec;
  if (a.tv_usec < 0) {
    a.tv_usec += 1e6;
    a.tv_sec--;
  }
  return a;
}
/*-----------------------------------------------------------------*/
double
DiffTimesToMSec(struct timeval a, struct timeval b)
{
  return (1e3 * (a.tv_sec - b.tv_sec) 
	  + 1e-3 *(a.tv_usec - b.tv_usec));
}
/*-----------------------------------------------------------------*/
unsigned int 
HashString(char *name, unsigned int hash, int endOnQuery)
{
  char *ch;

  for (ch = name; *ch != '\0'; ch++) {
    if (endOnQuery && (*ch == '?'))
      break;
    hash += (_rotl(hash, 19) + *ch);
  }
  return hash;
}

/*-----------------------------------------------------------------*/
unsigned int 
WrandHash(unsigned int url_hash, unsigned int svr_hash)
{
  unsigned url_digest, svr_digest;
  unsigned int combined_hval;

  /* only keep 31 bits */
  url_digest = url_hash & 0x7FFFFFFF;
  svr_digest = svr_hash & 0x7FFFFFFF;
  combined_hval = (1103515245 * ((1103515245 * svr_digest + 12345) ^ 
				 url_digest) + 12345) & 0x7FFFFFFF;
  return combined_hval;
}

/*-----------------------------------------------------------------*/
unsigned int 
WidespreadHash(unsigned int hash)
{
  hash += (hash * 0x62531965);
  hash = _rotl(hash, 21);
  return hash;
}

/*-----------------------------------------------------------------*/
void 
swap(ServerWeight *server_weight_, int left, int right)
{
  ServerWeight temp;
  temp = server_weight_[left];
  server_weight_[left] = server_weight_[right];
  server_weight_[right] = temp;
}
/*-----------------------------------------------------------------*/
void 
quickSort(ServerWeight *server_weight_, int left, int right)
{
  ServerWeight curWeight;
  int i, j;
  if (right > left) {
    curWeight = server_weight_[right];
    i = left - 1;
    j = right;
    while (1) {
      while (server_weight_[++i].weight > curWeight.weight)
	;
      while (j > 0) {
        if (server_weight_[--j].weight >= curWeight.weight)
          break;
      }
      if (i >= j)
        break;
      swap(server_weight_, i, j);
    }
    swap(server_weight_, i, right);
    quickSort(server_weight_, left, i-1);
    quickSort(server_weight_, i+1, right);
  }
}

/*----------------------------------------------------------------*/
typedef struct opt_svr {
  int server_sum;
  int opt_svr;
} OPT_SVR;

/* "optimal" replicas for total num of servers */
OPT_SVR opt_servers[] = {
  {4,    1},
  {8,    2},
  {16,   3},
  {32,   5},
  {64,   10},
  {128,  20},
  {256,  40},
  {512,  80},
  {1024, 160},
};

/*----------------------------------------------------------------*/
int 
GetOptReplicaNum(int total_server)
{
  int server_bin;
  int i;
  int opt_num;

  server_bin = sizeof(opt_servers)/sizeof(OPT_SVR);
  opt_num = opt_servers[0].opt_svr;
  for (i = 0; i < server_bin; i++) {
    if (total_server >= opt_servers[i].server_sum)
      opt_num = opt_servers[i].opt_svr;
    else
      break;
  }
  return opt_num;
}
/*----------------------------------------------------------------*/
#define TO_HOST_L(x)    x = ntohl(x);
#define TO_NETWORK_L(x) x = htonl(x);
/*----------------------------------------------------------------*/
void 
ntoh_NSQuery(NSQuery* p)
{
  TO_HOST_L(p->nsq_version);
  ntoh_NSQueryResult(&p->nsq_result);
  TO_HOST_L(p->nsq_randomToken);
  TO_HOST_L(p->nsq_delay);
  TO_HOST_L(p->nsq_id);          
  TO_HOST_L(p->nsq_startTime.tv_sec);
  TO_HOST_L(p->nsq_startTime.tv_usec);
}
/*----------------------------------------------------------------*/
void 
hton_NSQuery(NSQuery* p)
{
  TO_NETWORK_L(p->nsq_version);
  hton_NSQueryResult(&p->nsq_result);
  TO_NETWORK_L(p->nsq_randomToken);
  TO_NETWORK_L(p->nsq_delay);
  TO_NETWORK_L(p->nsq_id);          
  TO_NETWORK_L(p->nsq_startTime.tv_sec);
  TO_NETWORK_L(p->nsq_startTime.tv_usec);
}
/*----------------------------------------------------------------*/
void 
ntoh_NSQueryResult(NSQueryResult* p)
{
  TO_HOST_L(p->nsq_ttl);
}
/*----------------------------------------------------------------*/
void 
hton_NSQueryResult(NSQueryResult* p)
{
  TO_NETWORK_L(p->nsq_ttl);
}
/*----------------------------------------------------------------*/
void 
ntoh_LocalQueryInfo(LocalQueryInfo *p)
{
  TO_HOST_L(p->lqi_size);
  TO_HOST_L(p->lqi_id);
  TO_HOST_L(p->lqi_cache);
}
/*----------------------------------------------------------------*/
void 
hton_LocalQueryInfo(LocalQueryInfo *p)
{
  TO_NETWORK_L(p->lqi_size);
  TO_NETWORK_L(p->lqi_id);
  TO_NETWORK_L(p->lqi_cache);
}
/*----------------------------------------------------------------*/
void 
ntoh_LocalQueryResult(LocalQueryResult* p)
{
  TO_HOST_L(p->lq_id);
  TO_HOST_L(p->lq_ttl);
}
/*----------------------------------------------------------------*/
void 
hton_LocalQueryResult(LocalQueryResult* p)
{
  TO_NETWORK_L(p->lq_id);
  TO_NETWORK_L(p->lq_ttl);
}
/*----------------------------------------------------------------*/
void ntoh_CoDNSHeartbeatInfo(CoDNSHeartbeatInfo* p)
{
  TO_HOST_L(p->hi_sendTime.tv_sec);
  TO_HOST_L(p->hi_sendTime.tv_usec);
  TO_HOST_L(p->hi_versionNum);
  TO_HOST_L(p->hi_ID);
  TO_HOST_L(p->hi_statusVec);
  TO_HOST_L(p->hi_avgLocalLookupMS);
}
/*----------------------------------------------------------------*/
void hton_CoDNSHeartbeatInfo(CoDNSHeartbeatInfo* p)
{
  TO_NETWORK_L(p->hi_sendTime.tv_sec);
  TO_NETWORK_L(p->hi_sendTime.tv_usec);
  TO_NETWORK_L(p->hi_versionNum);
  TO_NETWORK_L(p->hi_ID);
  TO_NETWORK_L(p->hi_statusVec);
  TO_NETWORK_L(p->hi_avgLocalLookupMS);
}
