#ifndef _CODNS_LIB_H_
#define _CODNS_LIB_H_

#include <sys/time.h>

/* codnslib.h/codnslib.c is for library functions
   for codns. All context-independent functions reside here */

/* serve weight structure */
typedef struct ServerWeight {
  int          index;
  unsigned int weight;
} ServerWeight;

/* HRW releated */ 
int GetOptReplicaNum(int total_server);
void quickSort(ServerWeight *server_weight_, int left, int right);

/* hash functions */
unsigned int WidespreadHash(unsigned int hash);
unsigned int HashString(char *name, unsigned int hash, int endOnQuery);
unsigned int WrandHash(unsigned int url_hash, unsigned int svr_hash);

/* time functions */
struct timeval AddTimes(struct timeval a, struct timeval b);
struct timeval DiffTimes(struct timeval a, struct timeval b);
double DiffTimesToMSec(struct timeval a, struct timeval b);

/* average function */
double DecayAvg(double oldAvg, double sample, int *count, int countMax);

/*------------------------------------------------------------------------*/
/* useful macros */
#define __max(x, y)       ((x) > (y) ? (x) : (y))
#define __min(x, y)       ((x) < (y) ? (x) : (y))

#define NUMITEMS(x)       (sizeof(x)/sizeof(x[0]))

#define DoubleToInt(x)    ((int)((x) + 0.5))

#define IsTimeNegative(x) ((x).tv_sec < 0 || \
			  ((x).tv_sec == 0 && (x).tv_usec < 0))

#define ToMSec(x)         (int)((x).tv_sec * 1e3 + (x).tv_usec * 1e-3)

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

/*------------------------------------------------------------------------*/
#endif /* _CODNS_LIB_H_ */
