#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include "queue.h"
#include "codns.h"
#ifdef OS_DARWIN
#include "darwin_compat.h"
#endif
#if defined(OS_DARWIN) && !defined(OS_TIGER)
typedef int socklen_t;             
#endif

#ifndef max
#define max(x, y) ((x) > (y) ? (x) : (y))
#endif

#ifndef min
#define min(x, y) ((x) > (y) ? (x) : (y))
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 1
#endif

/* request info structure */
typedef struct RequestInfo {
  int ri_id;                       /* request id */
  int ri_reqsize;                  /* request size(HEADER+QUESTION) in bytes */
  char* ri_request;                /* copy of the request */
  struct timeval ri_timestamp;     /* timestamp at the time of request */
  struct sockaddr_in ri_addr;      /* addr of original requests */
  LIST_ENTRY(RequestInfo) ri_all;  /* for maintaining request info list */
  LIST_ENTRY(RequestInfo) ri_dead; /* for coalescing/removing dead requests */
}RequestInfo;

/* request list */
static LIST_HEAD(RequestList, RequestInfo)
     allRequests = LIST_HEAD_INITIALIZER(allRequests);
/*-----------------------------------------------------------------*/
#define TO_HOST_L(x) x = ntohl(x);
#define TO_NETWORK_L(x) x = htonl(x);
static void 
HtoN_LocalQueryInfo(LocalQueryInfo *p)
{
  TO_NETWORK_L(p->lqi_size);
  TO_NETWORK_L(p->lqi_id);
  TO_NETWORK_L(p->lqi_cache);
}
/*----------------------------------------------------------------*/
static void 
NtoH_LocalQueryResult(LocalQueryResult* p)
{
  TO_HOST_L(p->lq_id);
  TO_HOST_L(p->lq_ttl);
}
/*-------------------------------------------------------------------------*/
#if 0
static void
DebugPrint(const u_char* p) 
{
  /* takes the ptr of the DNS msessage
     and prints the question and the first answer section */
  const u_char* base = p;
  const u_char* t, *n;
  HEADER* hdr = (HEADER *)p;
  unsigned short ptr;

  fprintf(stderr, "qd:%d, an:%d ns:%d ar:%d\n", 
	  ntohs(hdr->qdcount), ntohs(hdr->ancount),
	  ntohs(hdr->nscount), ntohs(hdr->arcount));

  p += sizeof(HEADER);
  
  /* skip the question section */
  while (*p != 0) {
    ptr = *(unsigned short *)p;
    ptr = ntohs(ptr);

    if (ptr & 0xC000) {
      p += 1;
      break;
    } else {
      p += (*p + 1);
    }
  }
  p += 5;

  /* find the next */
  t = p;
  while (*p != 0) {
    ptr = *(unsigned short *)p;
    ptr = ntohs(ptr);

    if (ptr & 0xC000) { 
      p += 1; 
      break;
    } else {
      p += (*p + 1);
    }
  }
  n = p + 1;
  p = t;

  fprintf(stderr, "name:");
  while (*p != 0) {
    char label[64];

    ptr = *(unsigned short *)p;
    ptr = ntohs(ptr);
    if (ptr & 0xC000)  /* pointer */
      p = base + (ptr & 0x3FFF);

    memcpy(label, p+1, *p);
    label[*p] = 0;
    fprintf(stderr, "%s.", label);
    p += (*p + 1);
  }

  p = n;
  fprintf(stderr, "\n");

  {
    unsigned short temp;
    int ttl;
    struct in_addr addr;

    temp = *(unsigned short *)p;
    temp = ntohs(temp);
    fprintf(stderr, "type: %d\n", temp);
    p += 2;

    temp = *(unsigned short *)p;
    temp = ntohs(temp);
    fprintf(stderr, "class: %d\n", temp);
    p += 2;

    ttl = *(int *)p;
    ttl = ntohl(ttl);
    fprintf(stderr, "ttl: %d\n", ttl);
    p += 4;
    
    temp = *(unsigned short *)p;
    temp = ntohs(temp);
    fprintf(stderr, "rdlength: %d\n", temp);
    p += 2;
    
    addr.s_addr = *(unsigned int *)p;
    fprintf(stderr, "rddata: %s\n", inet_ntoa(addr));
    p += 4;
  }
}
#endif
/*-------------------------------------------------------------------------*/
static double
Elapsed(const struct timeval* t, const struct timeval* cur)
{
  return (cur->tv_sec - t->tv_sec + 
	  1e-6 * (cur->tv_usec - t->tv_usec));
}
/*-------------------------------------------------------------------------*/
static void 
FreeRequestInfo(RequestInfo* ri) 
{
  /* free the request entry */
  if (ri == NULL)
    return;

  LIST_REMOVE(ri, ri_all);
  if (ri->ri_request)
    free(ri->ri_request);
  free(ri);
}
/*-------------------------------------------------------------------------*/
static RequestInfo *
Lookup(int id) 
{
  /* find the request entry */
  RequestInfo* walk;

  LIST_FOREACH(walk, &allRequests, ri_all) {
    if (walk->ri_id == id)
      return walk;
  }
  return NULL;
}
/*-------------------------------------------------------------------------*/
static int
ParseQuestion(const char* p, char* name, unsigned short* qtype)
{
  /* parse the question section and
     return # of bytes for the question section */
#define FLAG_PTR (0xC0)
  int idx = 0;
  unsigned char len;
  unsigned int i;
  const char *base = p;

  while (TRUE) {

    len = *p;
    if ((len & FLAG_PTR) != 0) {
      fprintf(stderr, "parse error, unexpected pointer\n");
      return -1;
    } 
    p++;

    if (len == 0) { 
      /* we encountered a zero-length octect, 
	 which marks the end of the question section */
      *qtype = *(u_short *)p;
      *qtype = ntohs(*qtype);

      /* make it null-terminated */
      name[idx] = 0;

      return ((int)(p - base) + 4); /* +4 = sizoef(QTYPE) + sizeof(QCLASS) */
    } else {     
      /* read in the name */
      for (i = 0; i < len; i++) {
	if (idx >= MAXDNAME) {
	  fprintf(stderr, "parse error, unexpected pointer\n");
	  return -1;
	}
	name[idx++] = *p++;
      }
      name[idx++] = '.';
    }
  }

  return -1;
}
/*-------------------------------------------------------------------------*/
static void
AddToQueue(int id, struct sockaddr_in* addr, const char* req, int reqsize)
{
  RequestInfo* ri;

  if ((ri = calloc(1, sizeof(RequestInfo))) == NULL) {
    fprintf(stderr, "calloc failed\n");
    exit(-1);
  }
  ri->ri_id = id;
  ri->ri_addr = *addr;
  if (req) {
    ri->ri_reqsize = reqsize;
    ri->ri_request = calloc(1, reqsize);
    if (ri->ri_request == NULL) {
      fprintf(stderr, "calloc failed in allocaing the question section\n");
      exit(-1);
    }
    memcpy(ri->ri_request, req, reqsize);
  }
  gettimeofday(&ri->ri_timestamp, NULL);
  LIST_INSERT_HEAD(&allRequests, ri, ri_all);
}
/*-------------------------------------------------------------------------*/
static void
GetLDNSAddr(struct sockaddr_in* addr)
{
  static struct sockaddr_in* dns_addr;
  static int nscount = -1;

  if (nscount < 0) {
    int i;

    res_init();
    if (_res.nscount <= 0) {
      fprintf(stderr, "no nameservers found\n");
      exit(-1);
    }
    dns_addr = calloc(_res.nscount, sizeof(struct sockaddr_in));
    if (dns_addr == NULL) {
      fprintf(stderr, "calloc failed\n");
      exit(-1);
    }

    nscount = 0;
    for (i = 0; i < _res.nscount; i++) {
      if (_res.nsaddr_list[i].sin_addr.s_addr != htonl(INADDR_LOOPBACK))
	dns_addr[nscount++] = _res.nsaddr_list[i];
    }
    /* check for the loopback interface */
    if (nscount >= _res.nscount) {
      fprintf(stderr, "codns-loopback server not found\n");
      exit(-1);
    }
  }
  if (nscount <= 0) {
    fprintf(stderr, "no available nameserver\n");
    exit(-1);
  }

  /* want round-robin selection? then fix here */
  *addr = dns_addr[0];
}
/*-------------------------------------------------------------------------*/
static int
ForwardToLDNS(int s, char *req, int size)
{
  struct sockaddr_in addr;

  GetLDNSAddr(&addr);
  if (sendto(s, req, size, 0, (struct sockaddr*)&addr,
	     sizeof(struct sockaddr_in)) != size) {
    fprintf(stderr, "sendto failed\n");
    return -1;
  }
  return 0;
}
/*-------------------------------------------------------------------------*/
static int
ForwardToCoDNS(int s, int id, char* name)
{
  LocalQuery query;
  int size;

  memset(&query, 0, sizeof(query));
  size = strlen(name) + 1;          /* length of name */
  query.lq_info.lqi_size = size;
  query.lq_info.lqi_id = id;
  strcpy(query.lq_name, name);
  size += sizeof(query.lq_info) + sizeof(query.lq_zero);

  HtoN_LocalQueryInfo(&query.lq_info);
  if (write(s, &query, size) < 0)
    return -1;
  return 0;
}
/*-------------------------------------------------------------------------*/
static void
OnRequest(int lbSock, int exSock, int codnsSock)
{
#define QUERY_A_REC 1  
  char req[PACKETSZ], name[MAXDNAME+2];
  int addr_len, res;
  struct sockaddr_in addr;
  unsigned short qtype;
  HEADER *hdr;

  res = recvfrom(lbSock, req, sizeof(req), 0,
		 (struct sockaddr *)&addr, (socklen_t *)&addr_len);
  
  hdr = (HEADER *)req;
  if (hdr->qr) {
    /* answer was received, just ignore */
    fprintf(stderr, "weird, received an answer rather than a request\n");
  } else {
    int qsize = -1;

    /* parse the question secion only when it's a standard query */
    if (hdr->opcode == 0 && hdr->qdcount == htons(1))
      qsize = ParseQuestion(req + sizeof(HEADER), name, &qtype);

    if (qsize <= 0 || qtype != QUERY_A_REC) {
      ForwardToLDNS(exSock, req, res);
      AddToQueue(hdr->id, &addr, NULL, 0);
    } else {
      ForwardToCoDNS(codnsSock, hdr->id, name);
      AddToQueue(hdr->id, &addr, req, qsize + sizeof(HEADER));
    }
  }
}
/*-------------------------------------------------------------------------*/
static void
OnResponseFromCoDNS(int s, int lbSock)
{
  LocalQueryResult result;
  RequestInfo* orig;
  int res, size, hsize, i;
  union {
    HEADER hdr;
    u_char buf[NS_PACKETSZ];
  } response;
  u_char* p;
  unsigned short ptr;
  unsigned short ancount = 0;

  if ((res = read(s, &result, sizeof(result))) != sizeof(result)) {
    fprintf(stderr, "read from CoDNS failed res = %d\n", res);
    exit(-1);
  }
  NtoH_LocalQueryResult(&result);

  if ((orig = Lookup(result.lq_id)) == NULL) {
    fprintf(stderr, "request not found id = %d\n", result.lq_id);
    return;
  }
  if (!orig->ri_request) {
    fprintf(stderr, "something went wrong\n");
    FreeRequestInfo(orig);
    return;
  }

  /* make an answer response */
  hsize = sizeof(HEADER);
  memset(&response.hdr, 0, hsize);
  response.hdr.id = result.lq_id;
  response.hdr.qr = 1;
  response.hdr.rd = ((HEADER *)orig->ri_request)->rd;
  response.hdr.ra = ((HEADER *)orig->ri_request)->ra;
  response.hdr.qdcount = htons(1);

  for (ancount = 0; ancount < MAX_ANSWERS; ancount++)
    if (result.lq_address[ancount].s_addr == 0) 
      break;
  response.hdr.ancount = htons(ancount);

  /* copy the question section */
  p = response.buf + hsize;
  memcpy(p, orig->ri_request + hsize, orig->ri_reqsize - hsize);
  p += (orig->ri_reqsize - hsize);

  /* answer section */
  ptr = sizeof(HEADER);
  ptr |= 0xC000;
  for (i = 0; i < ancount; i++) {
    *(unsigned short *)p = htons(ptr); /* NAME : empty */
    p += 2;
    *(unsigned short *)p = htons(1);   /* TYPE : A record */
    p += 2;
    *(unsigned short *)p = htons(1);   /* CLASS : IN */
    p += 2;
    *(unsigned int *)p = htonl(result.lq_ttl); /* TTL : */
    p += 4;
    *(unsigned short *)p = htons(4);   /* RDLENGTH : 4 bytes */
    p += 2;
    *(unsigned int *)p = result.lq_address[i].s_addr; /* RDATA : ip address */
    p += 4;
  }
    
  /* send it to the original client */
  size = p - response.buf;
  if (sendto(lbSock, response.buf, size, 0, 
	     (struct sockaddr *)&orig->ri_addr,
	     sizeof(struct sockaddr_in)) != size) {
    fprintf(stderr, "sento failed in responding\n");
  }

  /* release the structure */
  FreeRequestInfo(orig);
}
/*-------------------------------------------------------------------------*/
static void
OnResponseFromLDNS(int s, int lbSock)
{
  struct sockaddr_in addr;
  int res, addr_len;
  union {
    HEADER hdr;
    u_char buf[NS_PACKETSZ];
  } response;
  RequestInfo* orig;

  if ((res = recvfrom(s, response.buf, sizeof(response.buf), 0,
		      (struct sockaddr *)&addr, (socklen_t *)&addr_len)) < 0) {
    fprintf(stderr, "read from LDNS failed res = %d\n", res);
    return;
  }
  if ((orig = Lookup(response.hdr.id)) == NULL) {
    fprintf(stderr, "request not found id = %d\n", response.hdr.id);
    return;
  }
  if (sendto(lbSock, response.buf, res, 0, 
	     (struct sockaddr *)&orig->ri_addr,
	     sizeof(struct sockaddr_in)) != res) {
    fprintf(stderr, "sento failed in responding\n");
  }
  FreeRequestInfo(orig);
}
/*-------------------------------------------------------------------------*/
static int 
PrepareUDPSocket(void)
{
  int sock;

  if ((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
    fprintf(stderr, "socket creation failed\n");
    exit(-1);
  }

  return sock;
}
/*-------------------------------------------------------------------------*/
static int
PrepareCoDNSSocket(void)
{
  int sock;
  struct sockaddr_in saddr;

  if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
    fprintf(stderr, "socket creation failed\n");
    exit(-1);
  }

  saddr.sin_family = AF_INET;
  saddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  saddr.sin_port = htons(CODNS_PORT);

  if (connect(sock, (struct sockaddr *) &saddr, 
	      sizeof(struct sockaddr_in)) < 0) {
    fprintf(stderr, "connection to CoDNS failed\n");
    close(sock);
    return(-1);
  }
  return sock;
}
/*-------------------------------------------------------------------------*/
static int
PrepareLoopbackSocket(void)
{
#define DNS_PORT 53
  struct sockaddr_in addr;
  int sock;

  if ((sock = PrepareUDPSocket()) < 0) {
    fprintf(stderr, "socket creation failed\n");
    exit(-1);
  }
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  addr.sin_port = htons(DNS_PORT);

  if (bind(sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
    fprintf(stderr, "Error in bind heartbeat socket %d at port %d \n", 
	    sock, DNS_PORT);
    exit(2);
  }
  return sock;
}
/*-------------------------------------------------------------------------*/
static void
PeriodicClean(const struct timeval* cur)
{
  RequestInfo* walk;
  LIST_HEAD(, RequestInfo) deadList = LIST_HEAD_INITIALIZER(deadList);

  LIST_FOREACH(walk, &allRequests, ri_all) {
    if (Elapsed(&walk->ri_timestamp, cur) > 30) {
      LIST_INSERT_HEAD(&deadList, walk, ri_dead);
    }
  }

  while ((walk = LIST_FIRST(&deadList)) != NULL) {
    LIST_REMOVE(walk, ri_dead);
    FreeRequestInfo(walk);
  }
}
/*-------------------------------------------------------------------------*/
int 
main(void)
{
  int lbSock, exSock, codnsSock, maxSock = 0;
  fd_set tempSet, masterSet;
  struct timeval nexttimeout;

  lbSock = PrepareLoopbackSocket();
  exSock = PrepareUDPSocket();
  codnsSock = PrepareCoDNSSocket();
  
  if (lbSock < 0 || exSock < 0 || codnsSock < 0) {
    fprintf(stderr, 
	    "socket creation failed lbSock=%d exSock=%d codnsSock=%d\n",
	    lbSock, exSock, codnsSock);
    exit(-1);
  }

  FD_ZERO(&masterSet);
  FD_SET(lbSock, &masterSet);
  maxSock = max(lbSock, maxSock);
  FD_SET(exSock, &masterSet);
  maxSock = max(exSock, maxSock);
  FD_SET(codnsSock, &masterSet);
  maxSock = max(codnsSock, maxSock);
  
  gettimeofday(&nexttimeout, NULL);
  nexttimeout.tv_sec += 30;

  while (1) {
    struct timeval timeout, cur;
    
    timeout.tv_sec = 30;
    timeout.tv_usec = 0;
    tempSet = masterSet;
    while (select(maxSock + 1, &tempSet, NULL, NULL, &timeout) < 0) {
      if (errno == EINTR) {
	fprintf(stderr, "EINTR in select\n");
	continue;
      }
      fprintf(stderr, "error in select - %d\n", errno);
      perror("select");
      exit(-1);
    }

    if (FD_ISSET(lbSock, &tempSet)) 
      OnRequest(lbSock, exSock, codnsSock);

    if (FD_ISSET(exSock, &tempSet)) 
      OnResponseFromLDNS(exSock, lbSock);

    if (FD_ISSET(codnsSock, &tempSet))
      OnResponseFromCoDNS(codnsSock, lbSock);

    /* clean outdated requests periodically */
    gettimeofday(&cur, NULL);
    if (cur.tv_sec > nexttimeout.tv_sec ||
	(cur.tv_sec == nexttimeout.tv_sec 
	 && cur.tv_usec >= nexttimeout.tv_sec)) {
      PeriodicClean(&cur);
      nexttimeout = cur;
      nexttimeout.tv_sec += 30;
    }
  }
  
  return 0;
}
