/*
 * $Id: gethostbyname.c,v 1.2 2004/12/23 01:35:42 kyoungso Exp $
 */

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <arpa/nameser.h>		/* XXX hack for _res */
#include <resolv.h>			/* XXX hack for _res */

#ifdef __sun
#include "strsep.c"
#endif

#define _PATH_HOSTCONF	"/etc/host.conf"

extern struct hostent *gethostbydnsnameX(const char *name, int af);
extern struct hostent *gethostbyhtnameX(const char *name, int af);

enum service_type {
  SERVICE_NONE = 0,
  SERVICE_BIND,
  SERVICE_HOSTS
};
#define SERVICE_MAX	SERVICE_HOSTS

static struct {
  const char *name;
  enum service_type type;
} service_names[] = {
  { "hosts", SERVICE_HOSTS },
  { "/etc/hosts", SERVICE_HOSTS },
  { "hosttable", SERVICE_HOSTS },
  { "htable", SERVICE_HOSTS },
  { "bind", SERVICE_BIND },
  { "dns", SERVICE_BIND },
  { "domain", SERVICE_BIND },
#if 0
  { "yp", SERVICE_NIS },
  { "yellowpages", SERVICE_NIS },
  { "nis", SERVICE_NIS },
#endif
  { 0, SERVICE_NONE }
};

static enum service_type service_order[SERVICE_MAX + 1];
static int service_done = 0;

static enum service_type
get_service_name(const char *name) {
  int i;
  for(i = 0; service_names[i].type != SERVICE_NONE; i++) {
    if(!strcasecmp(name, service_names[i].name)) {
      return service_names[i].type;
    }
  }
  return SERVICE_NONE;
}

static void
init_services()
{
  char *cp, *p, buf[BUFSIZ];
  register int cc = 0;
  FILE *fd;
  
  if ((fd = (FILE *)fopen(_PATH_HOSTCONF, "r")) == NULL) {
    /* make some assumptions */
    service_order[0] = SERVICE_BIND;
    service_order[1] = SERVICE_HOSTS;
    service_order[2] = SERVICE_NONE;
  } else {
    while (fgets(buf, BUFSIZ, fd) != NULL && cc < SERVICE_MAX) {
      if(buf[0] == '#')
	continue;
      
      p = buf;
      while ((cp = strsep(&p, "\n \t,:;")) != NULL && *cp == '\0')
	;
      if (cp == NULL)
	continue;
      do {
	if (isalpha((unsigned char)cp[0])) {
	  service_order[cc] = get_service_name(cp);
	  if(service_order[cc] != SERVICE_NONE)
	    cc++;
	}
	      while ((cp = strsep(&p, "\n \t,:;")) != NULL && *cp == '\0')
		;
      } while(cp != NULL && cc < SERVICE_MAX);
    }
    service_order[cc] = SERVICE_NONE;
    fclose(fd);
  }
  service_done = 1;
}

static struct hostent *
gethostbyname2X(const char *name, int type)
{
  struct hostent *hp = 0;
  int nserv = 0;
  
  if (!service_done)
    init_services();
  
  while (!hp) {
    switch (service_order[nserv]) {
    case SERVICE_NONE:
      return NULL;
    case SERVICE_HOSTS:
      hp = gethostbyhtnameX(name, type);
      break;
    case SERVICE_BIND:
      hp = gethostbydnsnameX(name, type);
      break;
#if 0
    case SERVICE_NIS:
      hp = _gethostbynisname(name, type);
      break;
#endif
    default:
      return NULL;
    }
    nserv++;
  }
  return hp;
}

struct hostent *
gethostbynameX(const char *name)
{
  return (gethostbyname2X(name, AF_INET));
}

